
/* PC1450 - A calculator emulator
 * Copyright (C) 1996 - 2001 Yves Maingoy
 * ymaingoy@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/***************************************************************************
    PROGRAM: pc1450.c

    PURPOSE: Emulateur du Sharp PC1450

    FUNCTIONS:

    WinMain() - calls initialization function, processes message loop
    InitApplication() - initializes window data and registers window
    InitInstance() - saves instance handle and creates main window
    MainWndProc() - processes messages
    About() - processes messages for "About" dialog box
    MakeColorBitmap(HWND) - creates a color bitmap
    MyTimerProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
    do_timer(HWND hWnd) - timer de l'emulation

    DIALOGS:

    DlgAbout(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
    DlgAllReset(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
    DlgConfigureBox(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
    DlgConfigureFilesBox(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
    DlgEmulateur(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
    DlgError(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
    DlgMemory(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)


    COMMENTS:
	Module principal de l application windows 32.

****************************************************************************/

#include "pc1450.h"
#include "pc1450res.h"
#include "video.h"
#include "memory.h"
#include "emule.h"
#include "keyboard.h"
#include "error.h"
#include "unassemble.h"

// Gestion windows
HINSTANCE hInst;		    // Instance handle
HWND hwndMain;		        // Main window handle

int isExecuteON = 0;        // Si 1 alors le timer est ON
int is_mode_info = 0;
long cptInstructions=0;

int cptLectureKbd;
//keyboard.c:
extern int isKeyReady;      // si une touche est disponible
extern int nvKey;           // valeur de la touche selon WM_CHAR
extern DWORD dwKeyData;     // autre valeur de la touche selon WM_CHAR

int  isEmulateurMsg = 0;    // Variable globale pour les messages de l'emulateur
char EmulateurMsg[128] ;    // Message de l'emulateur

BOOL bTrack = FALSE;        // TRUE if user is selecting a region
RECT Rect;
RECT RectTouche;
int  isRectTouche;

WORD wPrevItem;             // ancien item du menu RAM16, RAM8 ou RAM4




/****************************************************************************

    FUNCTION: InitApplication

    PURPOSE: Initializes window data and registers window class

****************************************************************************/
static BOOL InitApplication(void)
{
	WNDCLASS wc;

	memset(&wc,0,sizeof(WNDCLASS));
	wc.style = CS_HREDRAW|CS_VREDRAW |CS_DBLCLKS ;
	wc.lpfnWndProc = (WNDPROC)MainWndProc;
	wc.hInstance = hInst;
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);

	wc.lpszMenuName = "Pc1450Menu";                 // MAKEINTRESOURCE(IDMAINMENU);
	wc.lpszClassName = "Pc1450WClass";

    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;

	if (!RegisterClass(&wc))
		return 0;
	else
		return 1;
}




/****************************************************************************

    FUNCTION: CheckRamType

    PURPOSE: Change le menu selon le type de Ram

****************************************************************************/
void CheckRamType(HWND hwnd, WORD * wPrevItem)
{
    CheckMenuItem(GetMenu(hwnd), *wPrevItem, MF_UNCHECKED);
    switch(RamType)
    {
        case 16:
            *wPrevItem = IDM_RAM16;
            break;

        case 8:
            *wPrevItem = IDM_RAM8;
            break;

        default:
            *wPrevItem = IDM_RAM4;
    }
    CheckMenuItem(GetMenu(hwnd), *wPrevItem, MF_CHECKED);
}

/****************************************************************************

    FUNCTION: do_timer

    PURPOSE: Emulation d'une serie d'instructions
	NB: Il ne faut pas que le timer soit actif pendant l'excution
	des instructions. Pour cela, une variable globale est utilise.
	Celle-ci est initialise pendant le chargement de l'application.


****************************************************************************/
void do_timer(HWND hWnd)
{
    FARPROC lpProcEmulateur;
    int i;


	// Interdit le timer si l'execution est active !
	if (isExecuteON)
        return;
	else
		isExecuteON = 1;

    if ( NoROMLoaded ) {
		isExecuteON= 0;
        return;
    }
	else
	{
	    // On tente d'executer plusieurs instructions  la suite
	    for(i=0; i<10000; i++)
	    {
	        /* Gestion du clavier */
	        if (PC == 0x1330)
	        {
	            cptLectureKbd++;
	            if (cptLectureKbd>=3)
	            {
	                /* ici le scan code de la touche vient d'etre lu */
	                /* Efface donc la memoire des touche */
	                cptLectureKbd = 0;
	                KeyboardCross(0);
	            }
	        }

	        //cptInstructions++;
	        executeInstruction();

	        if(isEmulateurMsg!=0)
	        {
	            switch (isEmulateurMsg)
	            {
	                case 1:                 // message: rafraichir l'ecran
	                    isEmulateurMsg=0;
	                    if(isScreenRefresh != 0)
	                        PrintScreen(hWnd, CurrentY, &MemoryBuffer[0x7000]);
	                    isScreenRefresh = 0;
	                    break;

	                default:
	                    lpProcEmulateur = MakeProcInstance(DlgEmulateur, hInst);
	                    DialogBox(hInst, "EmulateurBox", hWnd, lpProcEmulateur);
	                    //FreeProcInstance(lpProcEmulateur);
	                    isEmulateurMsg=0;
	            }
				isExecuteON = 0;
	            return;
	        }
	    }
		// Autorise le timer
		isExecuteON = 0;
 	}
}

/****************************************************************************

    FUNCTION: ChangeRamType

    PURPOSE: Affiche une boite de dialogue pour changer le type de memoire

****************************************************************************/
void ChangeRamType(HWND hwnd, int valeur)
{
    FARPROC lpProcMemory;

	// interdit le timer
	isExecuteON = 1;

    if(valeur != RamType)
    {
        lpProcMemory = MakeProcInstance(DlgMemory, hInst);
        if(DialogBox(hInst, "MemoryBox", hwnd, lpProcMemory))
        {
            RamType = valeur;
            PC = 0x0000;
            nvKey = 0x00;
            isKeyReady=0;
            isTouchePresente = 0;
            isScreenRefresh = 1;
            CheckRamType(hwnd, &wPrevItem);
            UpdateWindow(hwnd);
         }
         //FreeProcInstance(lpProcMemory);
    }
	// Autorise le timer
	isExecuteON = 0;
}

/****************************************************************************

    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

****************************************************************************/
int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance, LPSTR lpCmdLine, INT nCmdShow)
{
	MSG msg;
	HANDLE hAccelTable;

	if (!InitApplication())
		return 0;


    // Cration de la fenetre
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

	// Autorise le timer
	isExecuteON = 0;

	while (GetMessage(&msg,NULL,0,0)) {
		//if (!TranslateAccelerator(msg.hwnd,hAccelTable,&msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		//}
	}
	return msg.wParam;
}


/****************************************************************************

    FUNCTION:  InitInstance

    PURPOSE:  Saves instance handle and creates main window

****************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
    HINSTANCE       hInstance;
    int             nCmdShow;
{
	hInst = hInstance;      // memoriser le pointeur principal

	//hAccelTable = LoadAccelerators(hInstance,MAKEINTRESOURCE(IDACCEL));

    hwndMain = CreateWindow("Pc1450WClass",
                            "Emulateur Pc1450 V  32 Beta 1.03",
                            WS_OVERLAPPED | WS_CAPTION |WS_SYSMENU
                            | WS_MINIMIZEBOX,
                            CW_USEDEFAULT,
                            CW_USEDEFAULT,
                            WHIDTHBMP ,         //int nWidth;
                            HEIGHTBMP+40,       //int nHeight;
                            NULL,
                            NULL,
                            hInst,
                            NULL
                           );

    if (!hwndMain)
        return (FALSE);
    else
    {
	    //CreateSBar(hwndMain,"Ready",1);

        ShowWindow(hwndMain, nCmdShow);
	    //ShowWindow(hwndMain, SW_SHOW);
        UpdateWindow(hwndMain);
        return (TRUE);
    }
}


/****************************************************************************

    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages

    MESSAGES: (non actualise)

    WM_COMMAND    -
        WM_CREATE      - create window and objects
        WM_KEYDOWN     -
        WM_LBUTTONDOWN -
        WM_MOUSEMOVE   -
        WM_LBUTTONUP   -
        WM_RBUTTONUP   -
        WM_ERASEBKGND  -
        WM_DESTROY     - destroy window

    COMMENTS:

    WM_DESTROY    - destroy window

****************************************************************************/

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message,
                             WPARAM wParam, LPARAM lParam)
{
    FARPROC lpProcConfigureBox;
    FARPROC lpProcAbout;
    FARPROC lpProcError;
    FARPROC lpProcAllReset;
    FARPROC lpProcUnassemble;
    TIMERPROC lpfnMyTimerProc;

    switch (message)
    {
        case WM_CREATE:                 // message: create window
            hEcranBitmap = LoadBitmap(hInst, "ecran");
            hPixelsBitmap = LoadBitmap(hInst, "pixels"); // Pointe sur la resource

            hDC = GetDC(hWnd);
            hMemoryDC = CreateCompatibleDC(hDC);
            ReleaseDC(hWnd, hDC);


            hOldBitmap = SelectObject(hMemoryDC, hBitmap1);
            GetObject(hBitmap1, 16, (LPSTR) &Bitmap);
            GetObject(hEcranBitmap, sizeof(BITMAP),(LPSTR) &bmEcran);   // Charge bmEcran avec la resource
            GetObject(hPixelsBitmap, sizeof(BITMAP),(LPSTR) &bmPixels); // Charge bmPixel avec la resource

            isTouchePresente = 0;
            cptLectureKbd = 0;
            isFatalError=0;
            LoadMemory(pc1450mem);        // lecture de la memoire
            if(!isFatalError) initRegs(); // Certains registres internes sont initialises ici
            switch(RamType)
            {
                case 16:
                    wPrevItem = IDM_RAM16;
                    break;
                case 8:
                    wPrevItem = IDM_RAM8;
                    break;
                default:
                    wPrevItem = IDM_RAM4;
                    break;
            }
            CheckRamType(hWnd, &wPrevItem);

            lpfnMyTimerProc = (TIMERPROC) MakeProcInstance((FARPROC)MyTimerProc, hInst);
            SetTimer(hWnd, ID_MYTIMER, 10, lpfnMyTimerProc);
            break;

        case WM_KEYDOWN:
            nvKey=0;
            isKeyReady=0;
            // ne lance l'interpretation des touches que si ce sont des touches de fonction
            switch (wParam)
            {
                case VK_F1:
                nvKey = 0x01;
                break;
                case VK_F2:
                nvKey = 0x02;
                break;
                case VK_F3:
                nvKey = 0x03;
                break;
                case VK_F4:
                nvKey = 0x04;
                break;
                case VK_F5:
                nvKey = 0x05;
                break;
                case VK_F6:
                nvKey = 0x06;
                break;
                case VK_F7:
                nvKey = 0x07;
                break;
                case VK_F8:
                nvKey = 0x08;
                break;
                case VK_F9:
                nvKey = 0x09;
                break;
                case VK_ESCAPE:
                nvKey = 0x32;
                break;
                case VK_SHIFT:
                nvKey = 0x0B;
                break;

                case VK_LEFT:
                nvKey = 0x09;
                break;
                case VK_RIGHT:
                nvKey = 0x0A;
                break;
                case VK_DOWN:
                nvKey = 0x07;
                break;
                case VK_UP  :
                nvKey = 0x08;
                break;
                case VK_BACK:
                nvKey = 0x09;
                break;
            }
            if( nvKey != 0)
            {
                isKeyReady=1;
                isTouchePresente = 0; // permet l'interpretation de la touche
                cptLectureKbd = 0;    // initialise les 3 passes necessaires pour l'interpretation
                return 0;
            }
            break;

        case WM_CHAR:                   // message: clavier
            if( isKeyReady==0 )
            {
                nvKey = AsciiToKey[wParam]; // Transforme le code nvKey
                if(nvKey!=0){
                    isKeyReady=1;
                    isTouchePresente = 0; // permet l'interpretation de la touche
                    cptLectureKbd = 0;    // initialise les 3 passes necessaires pour l'interpretation
                return 0;
                }
            }

        case WM_LBUTTONDOWN:            // message: left mouse button pressed
             // Recupre les coordonnes de la touche enfoncee
             isRectTouche = ToucheDown( hWnd, MAKEPOINTS(lParam), &RectTouche);
             if(isRectTouche!=0)
             {
                // Gnre le code de la touche enfoncee
                isKeyReady=1;
                isTouchePresente = 0; // permet l'interpretation de la touche
                cptLectureKbd = 0;    // initialise les 3 passes necessaires pour l'interpretation
                nvKey = isRectTouche;
             }
            break;

        case WM_LBUTTONUP:              // message: right mouse button released
            // si une touche est enfoncee, la relachee
            if(isRectTouche!=0)
            {
                ToucheUp(hWnd, RectTouche);
                isRectTouche = 0;
            }
            break;

        case WM_MOUSEMOVE:              // message: mouse movement
            // si une touche est enfoncee, la relachee
            if(isRectTouche!=0)
            {
                ToucheUp(hWnd, RectTouche);
                isRectTouche = 0;
            }
            break;

        //case WM_LBUTTONUP:              // message: left mouse button released

        case WM_ERASEBKGND:             // messsage: erase background
            isScreenRefresh = 1;
            AfficheEcran(hWnd); // Affiche l'ecran du sharp
            return TRUE;

        case WM_COMMAND: // Action sur les principales commandes de l'application
            switch (wParam)
            {
                case IDM_ABOUT:
                    lpProcAbout = MakeProcInstance(DlgAbout, hInst);
                    DialogBox(hInst, "AboutBox", hWnd,lpProcAbout);
                    //FreeProcInstance(lpProcAbout);
                    break;
                case IDM_INFO:
                    lpProcAbout = MakeProcInstance(DlgAbout, hInst);
                    DialogBox(hInst, "InfoBox", hWnd,lpProcAbout);
                    //FreeProcInstance(lpProcAbout);
                    break;

                case IDM_CONFIGFILES:
                    lpProcConfigureBox = MakeProcInstance(DlgConfigureFilesBox, hInst);
                    if(DialogBox(hInst, "CONFIGFILESBOX", hWnd, lpProcConfigureBox))
                    {

                    }
                    //FreeProcInstance(lpProcConfigureBox);
                    break;

                case IDM_SAUVEMEM:
                    SaveMemory(pc1450mem); // Ecriture de la memoire
                    break;

                case IDM_LOADMEM:
                    isTouchePresente = 0;
                    cptLectureKbd = 0;
                    isFatalError=0;
                    LoadMemory(pc1450mem); // lecture de la memoire
                    if(!isFatalError) initRegs(); // les registres internes sont initialises ici
                    CheckRamType(hWnd, &wPrevItem);
                    break;

                case IDM_QUITTER:
                    SendMessage(hWnd, WM_DESTROY, wParam, lParam);
                    break;

                case IDM_MAXDELAY:
                    lpProcConfigureBox = MakeProcInstance(DlgConfigureBox, hInst);
                    if(DialogBox(hInst, "CONFIGUREBOX", hWnd, lpProcConfigureBox))
                    {

                    }
                    //FreeProcInstance(lpProcConfigureBox);
                    break;

                case IDM_RAM16:
                    ChangeRamType(hWnd, 16);
                    break;
                case IDM_RAM8:
                    ChangeRamType(hWnd, 8);
                    break;
                case IDM_RAM4:
                    ChangeRamType(hWnd, 4);
                    break;

                case IDM_BREAK:
                    nvKey = 0x03;
                    isKeyReady=1;
                    isTouchePresente = 0; // permet l'interpretation de la touche
                    cptLectureKbd = 0;    // initialise les 3 passes necessaires pour l'interpretation
                    isScreenRefresh = 1;
                    break;
                case IDM_RESET:
                    PC = 0x0018;
                    isKeyReady=1;
                    nvKey = 0x03; //touche BRK
                    isTouchePresente = 0; // permet l'interpretation de la touche
                    cptLectureKbd = 0;    // initialise les 3 passes necessaires pour l'interpretation
                    isScreenRefresh = 1;
                    CheckRamType(hWnd, &wPrevItem);
                    UpdateWindow(hWnd);
                    break;
                case IDM_ALLRESET:
                    lpProcAllReset = MakeProcInstance(DlgAllReset, hInst);
                    if(DialogBox(hInst, "AllResetBox", hWnd,lpProcAllReset))
                    {
                        PC = 0x0000;
                        nvKey = 0x00;
                        isKeyReady=0;
                        isTouchePresente = 0;
                        isScreenRefresh = 1;
                        CheckRamType(hWnd, &wPrevItem);
                        UpdateWindow(hWnd);
                    }
                    //FreeProcInstance(lpProcAllReset);
                    break;

               case IDM_UNASSEMBLE:
                    lpProcUnassemble = MakeProcInstance(DlgUnassemble, hInst);
                    if(DialogBox(hInst, "UNASSEMBLEBOX", hWnd, lpProcUnassemble))
                    {
						unassemble();
                    }
                    break;
            }

            break;

        case WM_DESTROY:
            isFatalError=0;           // securite
            KillTimer(hWnd, ID_MYTIMER);
            SelectObject(hMemoryDC, hOldBitmap);
            DeleteDC(hMemoryDC);
            DeleteObject(hEcranBitmap);
            DeleteObject(hPixelsBitmap);
        PostQuitMessage(0);
        break;

        default:
            {
            return (DefWindowProc(hWnd, message, wParam, lParam));
            }
    }

    // Si une erreur fatale est survenue, quitter l'application
    if(isFatalError){
       isFatalError=0;
       lpProcError = MakeProcInstance(DlgError, hInst);
       DialogBox(hInst, "ErrorBox", hWnd,lpProcError);
       //FreeProcInstance(lpProcError);
       SendMessage(hWnd, WM_DESTROY, wParam, lParam);
    }

    return (0);
}

/****************************************************************************

    FUNCTION: DlgConfigureFilesBox(HWND, unsigned, WORD, LONG)

    PURPOSE:  "ConfigureBox" dialog box

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

****************************************************************************/
BOOL _stdcall DlgConfigureFilesBox(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static HWND hIDOK;

    switch (msg)
    {
        case WM_INITDIALOG:
        //"CONFIGFILESBOX"
            SetDlgItemText(hDlg, ID_FILERAM, pc1450mem);
            SetDlgItemText(hDlg, ID_FILELOAD, FileNameLOAD);
            SetDlgItemText(hDlg, ID_FILESAVE, FileNameSAVE);
            hIDOK=GetDlgItem(hDlg, IDOK);
            return TRUE;

        case WM_COMMAND:
            switch (wParam)
            {
                case IDOK:
                    // Prise des renseignements
                     GetDlgItemText(hDlg, ID_FILERAM, pc1450mem, 255);
                     GetDlgItemText(hDlg, ID_FILELOAD, FileNameLOAD, 255);
                     GetDlgItemText(hDlg, ID_FILESAVE, FileNameSAVE, 255);
                    EndDialog(hDlg, TRUE);
                    WriteIni();
                    return TRUE;

                case IDCANCEL:
                    EndDialog(hDlg, FALSE);
                    return TRUE;
            }
            break;
    }
    return FALSE;
}

/****************************************************************************

    FUNCTION: ConfigureDialog(HWND, unsigned, WORD, LONG)

    PURPOSE:  "ConfigureBox" dialog box

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

****************************************************************************/
BOOL _stdcall DlgConfigureBox(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static HWND hIDOK;

    switch (msg)
    {
        case WM_INITDIALOG:
            SetDlgItemInt(hDlg, ID_MAXDELAY, MaxDelay,    FALSE);
            SetDlgItemInt(hDlg, ID_MAX512, (int)(MaxHorloge512/100),    FALSE);
            SetDlgItemInt(hDlg, ID_MAX002, (int) MaxHorloge002,    FALSE);
            hIDOK=GetDlgItem(hDlg, IDOK);
            return TRUE;

        case WM_COMMAND:
            switch (wParam)
            {
                case IDDEFAUT:
                    SetDlgItemInt(hDlg, ID_MAXDELAY, (int)0,    FALSE);
                    SetDlgItemInt(hDlg, ID_MAX512, (int)200,    FALSE);
                    SetDlgItemInt(hDlg, ID_MAX002, (int) 20,    FALSE);
                    return TRUE;
                case IDOK:
                    // Prise des renseignements
                    MaxDelay      = GetDlgItemInt(hDlg, ID_MAXDELAY,  NULL, FALSE);
                    MaxHorloge512 = GetDlgItemInt(hDlg, ID_MAX512,    NULL, FALSE);
                    MaxHorloge512*= 100;
                    MaxHorloge002 = GetDlgItemInt(hDlg, ID_MAX002,    NULL, FALSE);
                    WriteIni();
                    EndDialog(hDlg, TRUE);
                    return TRUE;

                case IDCANCEL:
                    EndDialog(hDlg, FALSE);
                    return TRUE;


            }
            break;
    }
    return FALSE;
}

/****************************************************************************

    FUNCTION: DlgUnassemble

    PURPOSE:  Processes messages for "DlgUnassemble" dialog box

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

****************************************************************************/
BOOL _stdcall DlgUnassemble(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static HWND hIDOK;

    switch (msg)
    {
        case WM_INITDIALOG:
            SetDlgItemInt(hDlg, ID_ADR_START, (unsigned short)(unassemble_start),    FALSE);
            SetDlgItemInt(hDlg, ID_ADR_STOP, (unsigned short)(unassemble_stop), FALSE);
			SetDlgItemText(hDlg, ID_UNASSEMBLE_FILE, unassemble_file);
            hIDOK=GetDlgItem(hDlg, IDOK);
            return TRUE;

        case WM_COMMAND:
            switch (wParam)
            {
                case IDOK:
                    // Prise des renseignements
					unassemble_start = (unsigned short) GetDlgItemInt(hDlg, ID_ADR_START,  NULL, FALSE);
					unassemble_stop =  (unsigned short) GetDlgItemInt(hDlg, ID_ADR_STOP,  NULL, FALSE);
					unassemble_type =  (int)  IsDlgButtonChecked(hDlg, ID_CHK_ASSEMBLE);
                    GetDlgItemText(hDlg, ID_UNASSEMBLE_FILE, unassemble_file, 255);
					WriteIni();

                    EndDialog(hDlg, TRUE);
                    return TRUE;

                case IDCANCEL:
                    EndDialog(hDlg, FALSE);
                    return TRUE;
            }
            break;
    }
    return FALSE;
}


/****************************************************************************

    FUNCTION: DlgAbout

    PURPOSE:  Processes messages for "DlgAbout" dialog box

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

****************************************************************************/
BOOL _stdcall DlgAbout(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch (msg)
    {
        case WM_INITDIALOG:
            return (TRUE);

        case WM_COMMAND:
            if (wParam == IDOK || wParam == IDCANCEL)
            {
                EndDialog(hwnd, TRUE);
                return (TRUE);
            }
            break;
    }
    return (FALSE);
}

/****************************************************************************

    FUNCTION: DlgError(hDlg, message, wParam, lParam)

    PURPOSE:  Processes messages for "DlgError" dialog box

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

****************************************************************************/

BOOL _stdcall DlgError(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:
            SetDlgItemText(hDlg, IDC_TXTERROR, MsgError);
            return (TRUE);

        case WM_COMMAND:
            if (wParam == IDOK || wParam == IDCANCEL)
            {
                EndDialog(hDlg, TRUE);
                return (TRUE);
            }
            break;
    }
    return (FALSE);
}

/****************************************************************************

    FUNCTION: DlgEmulateur(hDlg, message, wParam, lParam)

    PURPOSE:  Processes messages for "DlgEmulateur" dialog box

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

****************************************************************************/
BOOL _stdcall DlgEmulateur(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:
            //SetDlgItemText(hDlg, IDC_TXTERROR, MsgError);
            return (TRUE);

        case WM_COMMAND:
            if (wParam == IDOK || wParam == IDCANCEL)
            {
                EndDialog(hDlg, TRUE);
                return (TRUE);
            }
            break;
    }
    return (FALSE);
}


/****************************************************************************

    FUNCTION: DlgAllReset(hDlg, message, wParam, lParam)

    PURPOSE:

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

****************************************************************************/
BOOL _stdcall DlgAllReset(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:
            return (TRUE);

        case WM_COMMAND:
            switch (wParam)
            {
                case IDOK:
                    EndDialog(hDlg, TRUE);
                    return (TRUE);
                break;

                case IDCANCEL:
                    EndDialog(hDlg, FALSE);
                    return (TRUE);
                break;
            }
            break;
    }
    return (FALSE);
}

/****************************************************************************

    FUNCTION: DlgAllReset(hDlg, message, wParam, lParam)

    PURPOSE:

    MESSAGES:

    WM_INITDIALOG - initialize dialog box
    WM_COMMAND    - Input received

****************************************************************************/
BOOL _stdcall DlgMemory(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:
            return (TRUE);

        case WM_COMMAND:
            switch (wParam)
            {
                case IDOK:
                    EndDialog(hDlg, TRUE);
                    return (TRUE);
                break;

                case IDCANCEL:
                    EndDialog(hDlg, FALSE);
                    return (TRUE);
                break;
            }
            break;
    }
    return (FALSE);
}



/****************************************************************************

    FUNCTION: MyTimerProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)

    PURPOSE: Initialize the timer

    COMMENTS:

****************************************************************************/
BOOL _stdcall MyTimerProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    do_timer(hwnd);
	return (0);
}


/****************************************************************************

    FUNCTION: MakeColorBitmap(HWND)

    PURPOSE: Creates a color bitmap

    COMMENTS:

        This creates a plaid color bitmap by using overlappying colors.

****************************************************************************/

HBITMAP MakeColorBitmap(hWnd)
HWND hWnd;
{
    HDC hDC;
    HDC hMemoryDC;
    HBITMAP hBitmap;
    HBITMAP hOldBitmap;
    HBRUSH hRedBrush;
    HBRUSH hGreenBrush;
    HBRUSH hBlueBrush;
    HBRUSH hOldBrush;

    hDC = GetDC(hWnd);
    hMemoryDC = CreateCompatibleDC(hDC);
    hBitmap = CreateCompatibleBitmap(hDC, 64, 32);
    hOldBitmap = SelectObject(hMemoryDC, hBitmap);
    hRedBrush = CreateSolidBrush(RGB(255,0,0));
    hGreenBrush = CreateSolidBrush(RGB(0,255,0));
    hBlueBrush = CreateSolidBrush(RGB(0,0,255));

    PatBlt(hMemoryDC, 0, 0, 64, 32, BLACKNESS);
    hOldBrush = SelectObject(hMemoryDC, hRedBrush);
    PatBlt(hMemoryDC, 0, 0, 24, 11, PATORDEST);
    PatBlt(hMemoryDC, 40, 10, 24, 12, PATORDEST);
    PatBlt(hMemoryDC, 20, 21, 24, 11, PATORDEST);
    SelectObject(hMemoryDC, hGreenBrush);
    PatBlt(hMemoryDC, 20, 0, 24, 11, PATORDEST);
    PatBlt(hMemoryDC, 0, 10, 24, 12, PATORDEST);
    PatBlt(hMemoryDC, 40, 21, 24, 11, PATORDEST);
    SelectObject(hMemoryDC, hBlueBrush);
    PatBlt(hMemoryDC, 40, 0, 24, 11, PATORDEST);
    PatBlt(hMemoryDC, 20, 10, 24, 12, PATORDEST);
    PatBlt(hMemoryDC, 0, 21, 24, 11, PATORDEST);

    SelectObject(hMemoryDC, hOldBrush);
    DeleteObject(hRedBrush);
    DeleteObject(hGreenBrush);
    DeleteObject(hBlueBrush);
    SelectObject(hMemoryDC, hOldBitmap);
    DeleteDC(hMemoryDC);
    ReleaseDC(hWnd, hDC);
    return (hBitmap);
}

/****************************************************************************

    FUNCTION: MakeLinePixel(HWND)

    PURPOSE: Creation d'un bitmap de 7*(3 + 1) points de haut sur 3 points de large

    COMMENTS:



****************************************************************************/
HBITMAP MakeLinePixel(hWnd)
HWND hWnd;
{
    HDC hDC;
    HDC hMemoryDC;
    HBITMAP hBitmap;
    HBITMAP hOldBitmap;
    HBRUSH hRedBrush;
    HBRUSH hGreenBrush;
    HBRUSH hBlueBrush;
    HBRUSH hOldBrush;

    hDC = GetDC(hWnd);
    hMemoryDC = CreateCompatibleDC(hDC);
    // taille : 7*(3+1) sur 3 * 128
    hBitmap = CreateCompatibleBitmap(hDC,3, 28);
    hOldBitmap = SelectObject(hMemoryDC, hBitmap);

    hRedBrush = CreateSolidBrush(RGB(255,0,0));
    hGreenBrush = CreateSolidBrush(RGB(0,255,0));
    hBlueBrush = CreateSolidBrush(RGB(0,0,255));

    PatBlt(hMemoryDC, 0, 0, 3,28, BLACKNESS);
    hOldBrush = SelectObject(hMemoryDC, hRedBrush);
    PatBlt(hMemoryDC, 0, 0, 3, 3, PATORDEST);
    PatBlt(hMemoryDC, 0, 4, 3, 3, PATORDEST);
    PatBlt(hMemoryDC, 0, 8, 3, 3, PATORDEST);
    SelectObject(hMemoryDC, hGreenBrush);
    PatBlt(hMemoryDC, 0,12, 3, 3, PATORDEST);
    PatBlt(hMemoryDC, 0,16, 3, 3, PATORDEST);
    SelectObject(hMemoryDC, hBlueBrush);
    PatBlt(hMemoryDC, 0,20, 3, 3, PATORDEST);
    PatBlt(hMemoryDC, 0,24, 3, 3, PATORDEST);

    SelectObject(hMemoryDC, hOldBrush);
    DeleteObject(hRedBrush);
    DeleteObject(hGreenBrush);
    DeleteObject(hBlueBrush);
    SelectObject(hMemoryDC, hOldBitmap);
    DeleteDC(hMemoryDC);
    ReleaseDC(hWnd, hDC);
    return (hBitmap);
}


